{ Solution to the problem 'Markov Trains'
  Joris van Rantwijk, May 2002. }

program Trains;

{ Input limits }
const
  MaxTrains = 100;
  MaxStation = 'L';

{ Problem description }
var
  NrTrains: Integer;
  TrainFrom: array[1..MaxTrains] of Char;
  TrainTo: array[1..MaxTrains] of Char;
  TrainFromTime: array[1..MaxTrains] of Integer;
  TrainToTime: array[1..MaxTrains] of Integer;
  TrainProb: array[1..MaxTrains] of Double;
  DepartStation, DestStation: Char;
  DepartTime, DestTime: Integer;
  AllStations: String;

{ Solution state }
var
  UsableTrains: Integer;
  TrainsByDepart: array[1..MaxTrains] of Integer;
  TrainsByArrive: array[1..MaxTrains] of Integer;
  ProbInTrain: array[1..MaxTrains] of Double;
  BestRoute: String;
  BestRouteProb: Double;


{ Convert a hh:mm time string to the number of minutes from midnight. }
function TimeFromStr(const s: String): Integer;
var
  h, m: Integer;
begin
  if (Length(s) <> 5) or (s[3] <> ':') then begin
    WriteLn(stderr, 'Invalid time format');
    Exit(1);
  end; {if}
  Val(Copy(s, 1, 2), h);
  Val(Copy(s, 4, 2), m);
  TimeFromStr := 60*h + m;
end; {TimeFromStr}


{ Chop the first word off s and return it. }
function GetWord(var s: String): String;
var
  i, j: Integer;
begin
  i := 1;
  while (i <= Length(s)) and (s[i] = ' ') do
    Inc(i);
  j := 1;
  while (i+j <= Length(s)) and (s[i+j] <> ' ') do
    Inc(j);
  GetWord := Copy(s, i, j);
  s := Copy(s, i+j, Length(s));
end; {GetWord}


{ Read a problem instance from the input. }
procedure ReadProblem;
var
  i, code: Integer;
  s, a, ta, b, tb, p: String;
begin
  { Read number of trains }
  ReadLn(NrTrains);
  if (NrTrains <= 0) or (NrTrains > MaxTrains) then begin
    WriteLn(stderr, 'Invalid number of trains');
    Halt(1);
  end; {if}

  { Read time table}
  AllStations := '';
  for i := 1 to NrTrains do begin

    { Read one train }
    ReadLn(s);
    a := GetWord(s);
    ta := GetWord(s);
    b := GetWord(s);
    tb := GetWord(s);
    p := GetWord(s);

    { Verify station names }
    if (Length(a) <> 1) or (Length(b) <> 1) then begin
      WriteLn(stderr, 'Invalid station name');
      Halt(1);
    end; {if}
    if (a[1] < 'A') or (a[1] > MaxStation) or
       (b[1] < 'A') or (b[1] > MaxStation) then begin
      WriteLn(stderr, 'Station name out of range');
      Halt(1);
    end; {if}
    if Pos(a, AllStations) = 0 then AllStations := AllStations + a;
    if Pos(b, AllStations) = 0 then AllStations := AllStations + b;

    { Store in table }
    TrainFrom[i] := a[1];
    TrainFromTime[i] := TimeFromStr(ta);
    TrainTo[i] := b[1];
    TrainToTime[i] := TimeFromStr(tb);
    Val(p, TrainProb[i], code);
    if (code <> 0) or (TrainProb[i] < 0.0) or (TrainProb[i] >= 1.0) then begin
      WriteLn(stderr, 'Invalid probability');
      Halt(1);
    end; {if}

    { Verify }
    if (TrainFrom[i] = TrainTo[i]) or 
       (TrainFromTime[i] >= TrainToTime[i]) then begin
      WriteLn(stderr, 'Invalid train service');
      Halt(1);
    end; {if}

  end; {for}

  { Read user query }
  ReadLn(s);
  a := GetWord(s);
  ta := GetWord(s);
  b := GetWord(s);
  tb := GetWord(s);
  if (Length(a) <> 1) or (Length(b) <> 1) then begin
    WriteLn(stderr, 'Invalid query format');
    Halt(1);
  end; {if}
  DepartStation := a[1];
  DepartTime := TimeFromStr(ta);
  DestStation := b[1];
  DestTime := TimeFromStr(tb);
  if (a[1] < 'A') or (a[1] > MaxStation) or
     (b[1] < 'A') or (b[1] > MaxStation) or
     (a = b) or (DepartTime >= DestTime) then begin
    WriteLn(stderr, 'Invalid query');
    Halt(1);
  end; {if}
end; {ReadProblem}


{ Recursively process all routes starting with r.
  Trains to this station have already been setup with the correct probabilities. }
procedure SolveRecurse(const r: String);
var
  curs, prevs: Char;
  prob: Double;
  i, j, t, ti, tj: Integer;
  b: Char;
  havedepart: boolean;
begin
  { Get current and previous station }
  curs := r[Length(r)];
  if Length(r) > 1 then
    prevs := r[Length(r)-1]
  else
    prevs := ' ';

  { Is this already the destination }
  if curs = DestStation then begin

    { Process arrival events in time order to get accumulated probability }
    prob := 0.0;
    if curs = DepartStation then
      prob := 1.0;
    for i := 1 to UsableTrains do begin
      t := TrainsByArrive[i];
      if (TrainFrom[t] = prevs) and (TrainTo[t] = curs) and (TrainToTime[t] <= DestTime) then
        prob := prob + ProbInTrain[t];
    end; {for}

    { Register this solution }
{DBG} WriteLn(stderr, 'DBG possible route=', r, ' prob=', prob:1:8);
{DBG} if (BestRouteProb / prob > 0.999) and (BestRouteProb / prob < 1.001) then
{DBG}   WriteLn(stderr, 'WARN: Probability ', prob:1:8, ' for route ', r, ' is close to ', BestRouteProb:1:8, ' for ', BestRoute);
    if prob > BestRouteProb then begin
      BestRoute := r;
      BestRouteProb := prob;
    end; {if}

  end {if}
  else begin

    { For each possible next station }
    for t := 1 to Length(AllStations) do begin
      b := AllStations[t];
      if Pos(b, r) = 0 then begin

        { Departure station starts at probability 1.0, rest at 0.0 }
        prob := 0.0;
	if curs = DepartStation then
	  prob := 1.0;

{DBG} WriteLn(stderr, 'DBG ', r,' (', b, ') ');

        havedepart := false;

        { Process departure and arrival events in time order }
	i := 1;
	j := 1;
	while (i <= UsableTrains) or (j <= UsableTrains) do begin
	  ti := 0;
	  if i <= UsableTrains then ti := TrainsByDepart[i];
	  tj := 0;
	  if j <= UsableTrains then tj := TrainsByArrive[j];

          { Select first event, prefer departure over arrival }
	  if (ti <> 0) and ((tj = 0) or (TrainFromTime[ti] <= TrainToTime[tj])) then begin

	    Inc(i);
	    if (TrainFrom[ti] = curs) and (TrainTo[ti] = b) then begin
	      { Handle a departure event }
	      ProbInTrain[ti] := prob * (1 - TrainProb[ti]);
	      prob := prob * TrainProb[ti];
	      if ProbInTrain[ti] > 0.0 then
	        havedepart := true;
{DBG} WriteLn(stderr, 'DBG depart ', ti, ' with prob=', ProbInTrain[ti]:1:8);
	    end; {if}

	  end {if}
	  else begin

	    Inc(j);
	    if (TrainFrom[tj] = prevs) and (TrainTo[tj] = curs) then begin
	      { Handle an arrival event }
	      prob := prob + ProbInTrain[tj];
{DBG} WriteLn(stderr, 'DBG arrive ', tj, ' with prob=', ProbInTrain[tj]:1:8, ' -> ', prob:1:8);
	    end; {if}

	  end; {if}
	end; {while}

        { Recursively solve all extensions of this route }
	if havedepart then
	  SolveRecurse(r+b);

      end; {if}
    end; {for}
  end; {if}
end; {SolveRecurse}


{ Find the best route from start to destination. }
procedure SolveProblem;
var
  i, j, t: Integer;
begin
  { Scan useable trains (in our timeframe) and sort them by time }
  UsableTrains := 0;
  for i := 1 to NrTrains do begin
    if (TrainFromTime[i] >= DepartTime) {and (TrainToTime[i] <= DestTime)} then begin

      Inc(UsableTrains);

      { Sort by departure time }
      j := UsableTrains;
      t := TrainFromTime[i];
      while (j > 1) and (t < TrainFromTime[TrainsByDepart[j-1]]) do begin
        TrainsByDepart[j] := TrainsByDepart[j-1];
	Dec(j);
      end; {while}
      TrainsByDepart[j] := i;

      { Sort by arrival time }
      j := UsableTrains;
      t := TrainToTime[i];
      while (j > 1) and (t < TrainToTime[TrainsByArrive[j-1]]) do begin
        TrainsByArrive[j] := TrainsByArrive[j-1];
	Dec(j);
      end; {while}
      TrainsByArrive[j] := i;

    end; {if}
  end; {for}

{DBG} Write(stderr, 'DBG trains by departure: ');
{DBG} for i := 1 to UsableTrains do
{DBG}   Write(stderr, TrainsByDepart[i], ' ');
{DBG} WriteLn(stderr);
{DBG} Write(stderr, 'DBG trains by arrival: ');
{DBG} for i := 1 to UsableTrains do
{DBG}   Write(stderr, TrainsByArrive[i], ' ');
{DBG} WriteLn(stderr);

  { Reset solution }
  BestRoute := '';
  BestRouteProb := 0.0;

  { Process all routes }
  SolveRecurse(DepartStation);
end; {SolveProblem}


{ Print the best route to the output. }
procedure WriteAnswer;
var
  i: Integer;
begin
  for i := 1 to Length(BestRoute) do begin
    Write(BestRoute[i]);
    if i < Length(BestRoute) then
      Write(' ');
  end; {for}
  WriteLn;
  WriteLn(BestRouteProb:1:4);
  if Round(BestRouteProb * 10000 - 0.1) <> Round(BestRouteProb * 10000 + 0.1) then
    WriteLn(stderr, 'WARN: Ambiguous rounding for prob=', BestRouteProb:1:8);
end; {WriteAnswer}


{ Main program }
var
  runs, i: Integer;
begin
  ReadLn(runs);
  for i := 1 to runs do begin
    ReadProblem;
    SolveProblem;
    WriteAnswer;
  end; {for}
end. {Trains}
